package App::Ping903::Command::nagios;
use strict;
use warnings;
use parent 'App::Ping903::Command';
use App::Ping903::Command ':exit_codes';
use File::Spec;
use File::Basename;
use Nagios::Config;

sub new {
    my ($class, $com, $agent) = @_;
    
    bless $class->SUPER::new($com, $agent,
			     'progname|p=s@' => 'prognames',
			     'config|c=s' => 'nagiosconf');
}

sub run {
    my $self = shift;
    $self->SUPER::run;
    $self->usage_error("extra parameters") if @ARGV;
    my $cf = $self->{options}{nagiosconf} // '/etc/nagios/nagios.cfg';
    push @{$self->{options}{prognames}}, 'ping903q';
    $self->{cfg} = Nagios::Config->new(filename => $cf);

    my %iph;
    foreach my $host ($self->{cfg}->list_hosts) {
	if ($host->register && $self->check_command($host)) {
	    $iph{$host->address} = 1;
	}
    }
    foreach my $service ($self->{cfg}->list_services) {
	if ($service->register && $self->check_command($service)) {
	    my @addr = map { $_->address } (@{$service->host_name});
	    @iph{@addr} = (1) x @addr
	}
    }

    my @iplist = keys %iph;
    my $n = @iplist;
    $self->error("info: collected $n addresses");
    unless ($self->agent->set_ip_list([@iplist])) {
	$self->abend(EX_FAIL, $self->agent->error_message);
    }
}

sub check_command {
    my ($self, $host) = @_;
    my $chk = $host->check_command or return;
    $chk =~ s/!.*//;
    my $def = $self->{cfg}->find_object($chk) or return;
    return unless $def->can('command_line');
    my ($cmd) =  split(/\s+/, $def->command_line);
    return grep {
		File::Spec->file_name_is_absolute($_)
		    ? $cmd eq $_
		    : basename($cmd) eq $_
	    } @{$self->{options}{prognames}};
}

1;

